/* Copyright (c) 2023 Renmin University of China
RMDB is licensed under Mulan PSL v2.
You can use this software according to the terms and conditions of the Mulan PSL v2.
You may obtain a copy of Mulan PSL v2 at:
        http://license.coscl.org.cn/MulanPSL2
THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
See the Mulan PSL v2 for more details. */

#pragma once

#include <cassert>
#include <cstring>
#include <memory>
#include <string>
#include <vector>

#include "parser/parser.h"
#include "system/sm.h"
#include "common/common.h"

class Query{
    public:
    std::shared_ptr<ast::TreeNode> parse;
    // TODO jointree
    // where条件
    std::vector<Condition> conds;
    // 投影列
    std::vector<TabCol> cols;
    // 表名
    std::vector<std::string> tables;
    // update 的set 值
    std::vector<SetClause> set_clauses;
    //insert 的values值
    std::vector<Value> values;

    Query(){}

};

class Analyze
{
private:
    SmManager *sm_manager_;
public:
    Analyze(SmManager *sm_manager) : sm_manager_(sm_manager){}
    ~Analyze(){}

    std::shared_ptr<Query> do_analyze(std::shared_ptr<ast::TreeNode> root);

private:
    TabCol check_column(const std::vector<ColMeta> &all_cols, TabCol target);
    void get_all_cols(const std::vector<std::string> &tab_names, std::vector<ColMeta> &all_cols);
    void get_clause(const std::vector<std::shared_ptr<ast::BinaryExpr>> &sv_conds, std::vector<Condition> &conds);
    void check_clause(const std::vector<std::string> &tab_names, std::vector<Condition> &conds);
    Value convert_sv_value(const std::shared_ptr<ast::Value> &sv_val);
    CompOp convert_sv_comp_op(ast::SvCompOp op);
    std::tm is_datetime(std::string str)
    {
        std::tm tm = {};
        if (strptime(str.c_str(), "%Y-%m-%d %H:%M:%S", &tm) == nullptr)
        {
            std::tm tm1{};
            tm1.tm_year = -2333;
            return tm1;
        }
        if (str.length() != std::string("1000-01-01 00:00:00").length())
        {
            std::tm tm1{};
            tm1.tm_year = -2333;
            return tm1;
        }
        if (str < "1000-01-01 00:00:00" || str > "9999-12-31 23:59:59")
        {
            std::tm tm1{};
            tm1.tm_year = -2333;
            return tm1;
        }
        int year = tm.tm_year + 1900;
        int month = tm.tm_mon + 1;
        int day = tm.tm_mday;
        int hour = tm.tm_hour;
        int minit = tm.tm_min;
        int sec = tm.tm_sec;


        // 检查月份范围（1月至12月）
        if (month < 1 || month > 12) {
            std::tm tm1{};
            tm1.tm_year = -2333;
            return tm1;
        }

        // 检查日期范围
        if (day < 1 || day > 31 || hour > 23 || minit > 59 || sec > 59) {
            std::tm tm1{};
            tm1.tm_year = -2333;
            return tm1;
        }

        // 检查大小月
        bool isLeapYear = ((year % 4 == 0 && year % 100 != 0) || (year % 400 == 0));
        bool isValidDay = false;

        if (month == 2) {  // 二月
            if (isLeapYear) {
                isValidDay = (day >= 1 && day <= 29);
            } else {
                isValidDay = (day >= 1 && day <= 28);
            }
        } else if (month == 4 || month == 6 || month == 9 || month == 11) {  // 小月（4、6、9、11月）
            isValidDay = (day >= 1 && day <= 30);
        } else {  // 大月（1、3、5、7、8、10、12月）
            isValidDay = (day >= 1 && day <= 31);
        }

        if (!isValidDay) {
            std::tm tm1{};
            tm1.tm_year = -2333;
            return tm1;
        }
        return tm;
    }
};

